"""The module that contains all component subclasses that can be used to generate Slide content

Classes
-------
#### SocialLink
Can be used to create a social media link icon, or just the icon

#### Link
A component for generating web links

#### Code
A component for adding code demos with syntax highlighting

#### Icon
A component that generates an icon

#### F‎ooter_
Allows you to add a footer to the presentation

#### Navbar
Allows you to add a navbar to the presentation

#### Button
A component that allows you to add html buttons

#### Raw
A component that dumps provided raw html

#### TableOfContents
A component used to generate table of contents for a presentation

#### Video
A component that allows you to embed a youtube video

#### Image
A component to include images

#### Grid
A component that allows you to evenly space multiple peices of content
"""
# Internal Dependencies
import enum
from abc import ABC
from typing import List, Union
from dataclasses import dataclass


class _Component(ABC):
    """Base class used for type checking, and inheritance on all components"""
    def __html__(self) -> str:
        raise NotImplementedError("Components require a __html__() method to be defined")


class SocialLink(enum.Enum):
    """Can be used to create a social media link icon, or just the icon

    Notes 
    -----
    - If you don't call the object's link function, it will just return the social media icon

    Examples
    -------
    ### Adding an icon link to someone's github in a navbar
    ```
    from ezprez.core import Presentation
    from ezprez.components import SocialLink, Navbar

    header = Navbar('Basic web technologies', [SocialLink.github.link("https://github.com/descent098")])
    Presentation(title, url, content, navbar=header)
    ```

    ### Adding an icon with no link to a Slide
    ```
    from ezprez.core import Slide
    from ezprez.components import SocialLink

    Slide('This is a youtube icon with no link', SocialLink.youtube)
    ```
    """
    url:str
    icon_color:str
    youtube = 0
    github = 1
    twitter = 2
    linkedin = 3
    twitch = 4


    def link(self, link):
        """Function used to asign URL parameter to the link

        Parameters
        ----------
        link : (str)
            The url you want to link to

        Returns
        -------
        The instance called from
        """
        self.url = link
        return self

    def color(self, color:str):
        """Function used to asign color parameter

        Parameters
        ----------
        color : (str)
            The color you want the icon to be

        Returns
        -------
        The instance called from
        """
        self.icon_color = color
        return self


    def __html__(self, header:bool = False) -> str:
        try: # If no icon_color is set
            self.__getattribute__("icon_color")
        except AttributeError:
            self.icon_color = "#141414"

        try: # If no url is set
            self.__getattribute__("url")
        except AttributeError:
            self.url = "#"

        if header:
            return f"""\n\t\t\t\t\t<a href='{self.url}' target='_blank' rel='external' style='background-color:#fff; color:{str(self.icon_color)};'>
                                <i class='fab fa-{self.name} fa-3x'></i>
                            </a>\n"""
        else:
            return f"""\n\t\t\t\t\t<a href='{self.url}' target='_blank' rel='external' style='color:{str(self.icon_color)};'>
                                <i class='fab fa-{self.name} fa-3x'></i>
                            </a>\n"""


@dataclass
class Link(_Component):
    """A component for generating web links

    Attributes
    ----------
    label : (str)
        The text shown that the link is applied to

    link : (str)
        The link to tie the label to

    Examples
    -------
    ### Add a link to google to a slide
    ```
    from ezprez.core import Slide
    from ezprez.components import Link

    Slide('This is a link to Google', Link('Click this to go to google', 'https://google.ca'))
    ```
    """
    label: str
    link: str
    color: str = "#4dd"


    def __html__(self, header:bool = False) -> str:
        if not header:
            return f"""\n\t\t\t\t\t<a href={self.link} target='_blank' rel='external' style='color:{self.color};'>{self.label}</a>\n"""
        else:
            return f"""\n\t\t\t\t\t<a href={self.link} target='_blank' rel='external' style='background-color:#fff; color:#141414;'>{self.label}</a>\n"""


@dataclass
class Code(_Component):
    """A component for adding code demos with syntax highlighting

    Attributes
    ----------
    language: (str)
        The programming language (Supports all the common languages listed on https://highlightjs.org/download/ )

    content: (str)
        The code to show off

    Notes
    -----
    - Uses highlightJS for syntax highlighting: https://highlightjs.org/download/

    Examples
    --------
    ### Add some html example code to a slide
    ```
    from ezprez.core import Slide
    from ezprez.components import Code

    Slide("Here is some example html", Code("html", "<p>This is some example code</p>"))
    ```
    """
    language: str
    content: str


    def _escape(self) -> str:
        """Escapes self.content so it's HTML safe

        Returns
        -------
        str
            The escaped string of content
        """
        self.content = self.content.replace("<", "&lt").replace(">", "&gt").replace("&", "&amp")
        return self.content


    def __html__(self) -> str:
        return f"""\t\t\t\t\t<pre><code class='language-{self.language.lower()}'>{self._escape()}</code></pre>\n"""


@dataclass
class Icon(_Component):
    """A component that generates an icon

    Attributes
    ----------
    label: (str)
        The icon name i.e. 'fa-heart' (details at https://fontawesome.com/)

    size: (str), optional;
        The size of icon, i.e. 60px, defaults to 48px

    Examples
    -------
    ### Put a heart icon in the middle of a slide
    ```
    from ezprez.core import Slide
    from ezprez.components import Icon

    Slide("This is a heart icon", Icon("fa-heart"))
    ```
    """
    label: str
    size: str = "48px"
    color: str = "#141414"


    def __html__(self) -> str:
        return f"""\t\t\t\t\t<svg class='{self.label}' style='width:{self.size};height:{self.size};color:{self.color};'><use xlink:href='#{self.label}'></use></svg>\n"""


@dataclass
class Footer(_Component):
    """Allows you to add a footer to the presentation

    Attributes
    ----------
    links: (list[Link or SocialLink])
        A list of Link's or SocialLink's to include in the footer

    Examples
    --------
    ### Adding a footer with a github link to a presentation
    ```
    from ezprez.core import Presentation
    from ezprez.components import Footer

    foot = Footer([SocialLink.github.link("https://github.com/Descent098/ezprez-example")])
    Presentation(title, description, url, footer=foot)
    ```
    """
    links: List[Union[Link, SocialLink]]


    def __html__(self) -> str:
        result =f"""
        <footer>
        """

        for link in self.links:
            if type(link) == Link or type(link) == SocialLink:
                result += f"\n\t\t\t{link.__html__()}\n"
            else:
                raise ValueError(f"Footer arguments must be of type ezprez.components.Link or ezprez.components.SocialLink, got \n\ttype {type(link)}\n\tValue{link}")

        result += """
        </footer>\n
        """
        return result


@dataclass
class Button(_Component):
    """A component that allows you to add html buttons

    Attributes
    ----------
    label: (str)
        The text that is shown on the button
    
    url: (str)
        The url the button links to
    
    icon: (False or Icon)
        Provide an Icon component to display in the button, optional and defaults to False
    
    ghost: (bool)
        Whether to not fill a color, optional and defaults to False
    
    rounding: (bool)
        Whether to round the corners of the button or not, optional and defaults to True
    
    color: (False or str)
        The color of the button, optional and defaults to False
    
    text_color: (False or str)
        The color of the button text, optional and defaults to False

    Notes 
    -----
    - The color and text_color attribute takes an HTML color as a string i.e. #f0f0f0, blue, rgba(255, 255, 255, 0.9)
    - If ghost is set to True all other colors will be ignored
    - The URL parameter also can use anchor links to other slides

    Examples
    --------
    ### Adding a button to a slide
    ```
    from ezprez.core import Slide
    from ezprez.components import Button

    b = Button("click me", "https://kieranwood.ca")
    Slide("Here's a button", b)
    ```
    """
    label: str # The text to put inside the button
    url: str # The url to link the button to
    icon: Union[bool, Icon] = False # The icon to put on the button (if you want one)
    ghost: bool = False # Whether to fill in a color or use ghost styling
    rounding: bool = True # Whether to round
    color: Union[bool, str] = False # The color to make the button
    text_color: Union[bool, str] = False # The color to make the button text

    def __html__(self) -> str:
        color_string = ""
        if self.color and self.text_color and not self.ghost:
            color_string = f"style='background-color:{self.color};color:{self.text_color};'"
        elif self.color or self.text_color and not self.ghost:
            color_string = "style="
            if self.color:
                color_string += f'\"background-color:{self.color};\"'
            if self.text_color:
                color_string += f'\"color:{self.text_color};\"'

        return f"""\n\t\t\t<a href='{self.url}' class='button{' ghost' if self.ghost else ''}{' radius' if self.rounding else ''}' {color_string}>{self.icon.__html__() if self.icon else ''} {self.label} </a>\n"""


@dataclass
class Navbar(_Component):
    """Allows you to add a navbar to the presentation

    Attributes
    ----------
    title: (str)
        The presentation title (used for accessibility reasons)

    links: (list[Link or SocialLink])
        A list of Link's or SocialLink's to include in the Navbar

    Examples
    --------
    ### Adding a Navbar with a github link to a presentation
    ```
    from ezprez.core import Presentation
    from ezprez.components import Navbar

    nav = Navbar('Presentation title', [SocialLink.github.link("https://github.com/Descent098/ezprez-example")])
    Presentation(title, description, url, navbar=nav)
    ```
    """
    title: str
    links: List[Union[Link, SocialLink]]


    def __html__(self) -> str:
        result =f"""
        <header role="banner">
            <nav role="navigation">
                <p class="logo"><a href="" target="_blank" title="{self.title}">{self.title}</a></p>
                <ul>
        """

        for link in self.links:
            if type(link) == Link or type(link) == SocialLink:
                result += f"\n\t\t\t\t\t<li>{link.__html__(header=True)}</li>\n"
            else:
                raise ValueError(f"Navbar arguments must be of type ezprez.components.Link or ezprez.components.SocialLink, got \n\ttype {type(link)}\n\tValue{link}")
        result += """
                </ul>
            </nav>
        </header>
        """

        return result


@dataclass
class Raw(_Component):
    """A component that dumps provided raw html

    Attributes
    ----------
    content: (str) 
        The HTML content to generate

    Examples
    -------
    ### Add a manual paragraph tag to a Slide
    ```
    from ezprez.core import Slide
    from ezprez.components import Raw

    content = '<p>This is some text</p>'

    Slide('This is some raw content', Raw(content))
    ```
    """
    content:str


    def __html__(self) -> str:
        return self.content

@dataclass
class TableOfContents(_Component):
    """A component used to generate table of contents for a presentation

    Attributes
    ----------
    sections: (dict)
        A dictionary of {label:slide_number}'s to generate the table of contents with

    Examples
    ----------
    ### Create a table of contents Slide for a presentation
    ```
    from ezprez.core import Slide
    from ezprez.components import TableOfContents

    # The argument is a dictionary where each key is the label in the table
    # and each value is the slide number
    sections = {'Intro':2, 'End': 5}

    Slide('Table of contents', TableOfContents(sections))
    ```
    """
    sections: dict


    def __html__(self) -> str:
        result = "\n\t\t<hr>\n\t\t<div class='toc'>\n\t\t\t<ol>"
        for section_title in self.sections:
            result += f"\n\t\t\t\t<li>\n\t\t\t\t\t<a href='#slide={self.sections[section_title]}' title='Go to {section_title}'>\n\t\t\t\t\t<span class='chapter'>{section_title}</span>\n\t\t\t\t\t<span class='toc-page'>{self.sections[section_title]}</span>\n\t\t\t\t</a></li>"
        result += "\n\t\t\t</ol>\n\t\t</div>\n"
        return result 


@dataclass
class Video(_Component):
    """A component that allows you to embed a youtube video

    Attributes
    ----------
    video_id: (str)
        The youtube video id, for example if the url is youtube.com/watch?v=wSVljLh1VmI then the id is 'wSVljLh1VmI'

    Examples
    --------
    ### Adding a youtube video to a Slide
    ```
    from ezprez.core import Slide
    from ezprez.components import Video

    Slide('Here is a video', Video('wSVljLh1VmI'))
    ```
    """
    video_id: str

    def __html__(self) -> str:
        return f"""\n\t\t\t\t<div class='embed'>\n\t\t\t\t\t<iframe src='https://www.youtube.com/embed/{self.video_id}' frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen ></iframe>\n\t\t\t\t</div> """

@dataclass
class Image(_Component):
    """A component to include images

    Attributes
    ----------
    title: (str) 
        The alternate text to describe what an image is

    filename: (str)
        The filename of the image

    browser: (bool)
        If true you can add a browser window border to the image https://webslides.tv/demos/components#slide=114

    width: (int or False)
        Used to override the width of an image, optional and defaults to False

    height: (int or False)
        Used to override the height of an image, optional and defaults to False

    Notes
    -----
    - All images must be included in the same directory as the presentation source file under /img or /images

    Examples
    --------
    ### Add an image to a slide
    ```
    from ezprez.core import Slide
    from ezprez.components import Image

    Slide("This is an image", Image("low poly ice caps", "kieran-wood-lp-ice-caps-4k-w-peng.jpg"))
    ```

    ### Add a background image to a slide
    ```
    from ezprez.core import Slide
    from ezprez.components import Image

    Slide("This is a background image", image=Image("low poly ice caps", "kieran-wood-abstract-landscape.jpg"), background="black")
    ```
    """
    title: str
    filename: str
    browser: bool = False
    width: Union[bool, int] = False
    height: Union[bool, int] = False

    def __html__(self) -> str:
        if not self.browser:
            return f"""<img src='./static/images/{self.filename}' alt='{self.title}'{f' width={self.width} ' if self.width else ''} {f' height={self.height} ' if self.height else ''}>"""
        else:
            return f"""<figure class='browser'><img src='./static/images/{self.filename}' alt='{self.title}'{f' width={self.width} ' if self.width else ''} {f' height={self.height} ' if self.height else ''}></figure>\n """


class Grid(_Component):
    """A component that allows you to evenly space multiple peices of content

    Attributes
    ----------
    contents: (Component, str, List[Component], List[str])
        The contents you want to include in the grid

    Examples
    --------
    ### Add a grid with content
    ```
    from ezprez.core import Slide
    from ezprez.components import Grid

    Slide("You can also have grids", Grid("With", "Lots", "of", "content"))
    ```

    ### Add a slide of a grid of lists
    ```
    from ezprez.core import Slide
    from ezprez.components import Grid

    Slide("Like, alot of content...", Grid(["You can stack content within grids", ["like this", "and this", "and even this"]], ["This is getting too much now", ["way", "way way", "too much"]]))
    ```
    """
    def __init__(self, *contents:Union[_Component, str, List[_Component], List[str]]):
        self.contents = contents

    def __html__(self):
        result = "\n\t\t\t\t<div class='grid'>"
        for content in self.contents:
            if type(content) == str:
                result += f"\n\t\t\t\t\t<div class='column'><p>{content}</p></div>\n"
            elif isinstance(content, _Component):
                result += "\n\t\t\t\t\t<div class='column'>"
                result += content.__html__()
                result += "</div>"
            elif type(content) == list:
                result += "\n\t\t\t\t\t<div class='column'>"
                for subcontent in content:
                    if isinstance(subcontent, _Component):
                        result += subcontent.__html__()
                    elif type(subcontent) == str:
                        result += f"\n\t\t\t\t\t<p>{subcontent}</p>\n"
                    elif type(subcontent) == list:
                        result += "\n\t\t\t\t\t<ul>"
                        for point in subcontent:
                            result += f"\n\t\t\t\t\t<li>{point}</li>\n"
                        result += "\n\t\t\t\t\t</ul>"
                result += "\n\t\t\t\t\t</div>"
            else:
                raise ValueError(f"Provided content to Grid was {type(content)}, which is not List, Component or string")
        return result


"""The module that contains the Presentation and Slide classes that are used to generate web presentations

Classes
-------
#### Slide
The class that is used to generate slides that are fed into the Presentation object

####Presentation
The class for defining the presentation configuration, and primary entrypoint to exporting presentations

Notes
-----
- On first run you will need an internet connection to download webslides

Examples
--------
#### Creating a presentation with a slide and exporting it to ./Presentation
```
from ezprez.core import Presentation, Slide

# This slide is auto-added to the Presentation object by default
Slide('This is the title', 'and this is the content')

prez = Presentation(title, description, url)

# Export the files to the current directory at /Presentation, and delete existing files if they're found
prez.export(".", force=True, folder_name="Presentation")
```
"""
# Standard lib dependencies
import os                                   # Used in path validation
from datetime import datetime               # Used to get date for export
from typing import Union, List              # Used to enrich type hints in methods
from shutil import copyfile, copytree, rmtree         # Used to do high level filesystem operations
from dataclasses import dataclass, field    # Used to make class generation faster and more efficient

# Internal dependencies
from ezprez.components import *             # Used for type checking in content generation
from ezprez.components import _Component    # Used for type checking in content generation

# External Dependencies
from tqdm import tqdm                       # Used for progress bars
from elevate import elevate                 # Used for any protected folder access such as system wide python installs or exports
from pystall.core import ZIPResource, build # Used to download webslides from the latest version and extract it for use on export


class Slide:
    """The class that is used to generate slides that are fed into the Presentation object

    Class Variables
    ---------------
    all: (List[Slide])
        Contains all slide instances that have been instatiated, accessed through Slide.all

    Attributes
    ----------
    heading: (str)
        The slide's title/heading

    contents: (str, list, or Component)
        An arbitrary number of strings, lists or Component subclasses that make up the contents of the slide

    background: (False or str)
        The background color of the slide (see notes), optional by default False

    image: (False or Image)
        An image to use as a background for the slide, optional by default False

    horizontal_alignment: (str)
        Where to align the slide contents horizontally, optional defaults to 'center'

    vertical_alignment: (str)
        Where to align the slide contents vertically, optional defaults to 'center'
    
    animation: (str)
        Which animation to use for slide transition, optional defaults to 'fadeIn'

    Notes
    -----
    - Horizontal alignment can be left, right or center
    - Vertical alignment can be top, bottom or center
    - Background colors available can be found at https://webslides.tv/demos/components#slide=27; just make sure to remove 'bg', so for example 'bg-black' becomes 'black'
    - Contents is an unpacked variable, meaning you just specify multiple sets of contents such as Slide(title, 'this is content', ['and', 'so', 'is', 'this'], Button('This to', '#'))
    - Animation can be fadeIn, fadeInUp, zoomIn, slideInLeft, slideInRight
    - You can also add 'slow' to the end of any animation to slow the animation i.e. 'zoomIn slow'


    Examples
    --------
    ### Create a basic text slide
    ```
    from ezprez.core import Slide
    from ezprez.components import Button

    Slide('Here is a text slide', 'and this is the text')
    ```

    ### Create a slide with some text, a list, and a button
    ```
    from ezprez.core import Slide
    from ezprez.components import Button

    Slide('This is the title...', '... and this is content', ['and', 'so', 'is', 'this'], Button('This to', '#'))
    ```
    """
    all = []

    def __init__(self, heading:str, *contents:Union[str, list, tuple, _Component], background:Union[bool,str] = False, horizontal_alignment:str = "center", vertical_alignment:str = "center", image:Union[bool, Image]=False, animation:str = "fadeIn" ):
        # Assign class attributes
        self.image = image
        self.heading = heading
        self.contents = contents
        self.background = background
        self.vertical_alignment = vertical_alignment 
        self.horizontal_alignment = horizontal_alignment
        self.animation = animation

        # Append slide to the class variable Slide.all
        Slide.all.append(self)

    def _generate_content(self):
        """Generates the necessary html with the provided contents"""
        result = f"\n\t\t\t<section class='bg-{self.background} slide-{self.vertical_alignment}'>"
        
        if self.image:
            result += f"\n\t\t\t\t<span class='background' style='background-image:url(\"./static/images/{self.image.filename}\")'></span>"
        result += f"\n\t\t\t\t<div class='wrap {self.animation}'>\n\t\t\t\t\t<div class='content-{self.horizontal_alignment}'>\n\t\t\t\t\t<h2>{self.heading}</h2>\n"
        
        for content in self.contents:
            if type(content) == str: # If the current peice of content is a str
                result += f"\t\t\t\t\t<p>{content}</p>\n"

            elif type(content) == list or type(content) == tuple: # If the current peice of content is a list or tuple
                result += f"\t\t\t\t\t<div class='grid {'content-' + self.horizontal_alignment if not self.horizontal_alignment == 'right' else ''}'>\n\t\t\t\t\t\t\t<ul style='text-align:justify;'>\n"
                for bullet_point in content: # Iterate through each element in the list/tuple
                    result += f"\t\t\t\t\t\t\t\t<li>{bullet_point}</li>\n"
                result += "\t\t\t\t\t</div>\n\t\t\t\t\t\t\t</ul>\n"
            elif isinstance(content, _Component) or type(SocialLink): # If the current peice of content is a Component
                result += content.__html__()
        result += "\n\t\t\t\t\t</div>\n\t\t\t\t</div>\n\t\t\t</section>\n"

        yield result


    def __html__(self):
        """Generates the markup for each slide"""
        result = ""
        for content in self._generate_content():
            result+= content
        return result


@dataclass
class Presentation:
    """The class for defining the presentation configuration, and primary entrypoint to exporting presentations

    Attributes
    ----------
    title: (str)
        The title of the presentation

    description: (str)
        The description of the presentation

    url: (str)
        The canonical URL the presentation will be deployed at

    slides: List[Slide]
        The slides to generate the presentation with, optional and defaults to Slide.all

    background: (str)
        What color the intro slie and default slide background color should be, optional defaults to 'white'

    image: (Image or False)
        Path to background image to use for og tags, and intro slide, optional defaults to False

    endcard: (bool)
        Whether to add the endcard, optional defaults to True

    intro: (bool)
        Generate an introduction slide with the background image, title, and description, optional defaults to True

    favicon: (Image or False)
        The image to use for the favicon, optional defaults to False

    vertical: (bool)
        Whether the slideshow should be vertical or not, optional defaults to False

    navbar: (Navbar or False)
        The navbar for the site, optional defaults to False

    footer: (Footer or False)
        The footer for the site, optional defaults to False

    Methods
    -------
    export:
        Exports the presentation files

    Examples
    --------
    ### Creating a presentation and exporting it to ./Presentation
    ```
    from ezprez.core import Presentation

    prez = Presentation(title, description, url)

    # Export the files to the current directory at /Presentation, and delete existing files if they're found
    prez.export(".", force=True, folder_name="Presentation")
    ```

    ### Creating a presentation with a slide and exporting it to ./Presentation
    ```
    from ezprez.core import Presentation, Slide

    # This slide is auto-added to the Presentation object by default
    Slide('This is the title', 'and this is the content')

    prez = Presentation(title, description, url)

    # Export the files to the current directory at /Presentation, and delete existing files if they're found
    prez.export(".", force=True, folder_name="Presentation")
    ```
    """
    title: str 
    description: str 
    url: str
    intro: bool = True
    endcard: bool = True
    vertical: bool = False
    background: str = "white"
    image: Union[bool, Image] = False
    favicon: Union[bool, Image] = False
    navbar: Union[bool, Navbar] = False
    footer: Union[bool, Footer] = False
    slides: List[Slide] = field(default_factory=lambda: Slide.all) 


    def _generate_favicon_markup(self):
        """Generates the html to render the favicon properly"""
        if self.favicon:
            return f'''<!-- FAVICONS -->
            <link rel="apple-touch-icon icon" sizes="76x76" href="./static/images/{self.favicon.filename}">'''
        else:
            return f'''<!-- FAVICONS -->
            <link rel="apple-touch-icon icon" sizes="76x76" href="static/images/favicons/favicon-152.png">'''


    def _generate_intro_slide(self):
        """Generates the first slide in a presentation"""
        if self.intro:
            if self.image:
                return f"""\t\t\t<section class='bg-{self.background}'>
                <span class='background' style='background-image:url("./static/images/{self.image.filename}")'></span>
                    <div class='wrap aligncenter'>
                        <h1><strong>{self.title}</strong></h1>
                        <p class='text-intro'>{self.description}</p>
                    </div>
            </section>"""
            else:
                return f"""\t\t\t<section class='bg-{self.background}'>
                    <div class='wrap aligncenter'>
                        <h1><strong>{self.title}</strong></h1>
                        <p class='text-intro'>{self.description}</p>
                    </div>
            </section>"""


    def _generate_endcard(self) -> str:
        """Generates the slide that goes at the end of Presentation's"""
        if self.endcard:
            return f"""\t\t\t<section class='bg-{self.background}'>
                    <div class='wrap aligncenter'>
                        <h2><strong>Thank you</strong></h2>
                        <h4 class='text-intro'>This presentation was made with the help of</h4>
                        <h4>
                            <strong><em><a href='https://webslides.tv' target='_blank'>WebSlides</a></em></strong> 
                        </h4>
                        <h4>
                            <strong><em><a href='https://github.com/Descent098/ezprez' target='_blank'>ezprez</a></em></strong>
                        </h4>
                    </div>
            </section>"""
        else:
            return ""


    def __len__(self) -> int:
        """Returns the number of slides in the presentation"""
        return len(self.slides)


    def __html__(self) -> str:
        """Generates the index.html file of a presentation using the provided slides"""
        slides_html = ""
        slide_iterator = tqdm(self.slides)
        slide_iterator.set_description_str("Generating slide content")
        for slide in slide_iterator:
            if not slide.background:
                slide.background = self.background
            slides_html += slide.__html__()

        result = f'''<!doctype html>
<html lang="en" prefix="og: http://ogp.me/ns#">
    <head>
        <meta charset="utf-8">
        <meta name="viewport" content="width=device-width, initial-scale=1">

        <!-- SEO -->
        <title>{self.title}</title>
        <meta name="description" content="{self.description}">

        <!-- URL CANONICAL -->
        <link rel="canonical" href="{self.url}">

        <!-- Google Fonts -->
        <link href="https://fonts.googleapis.com/css?family=Roboto:100,100i,300,300i,400,400i,700,700i%7CMaitree:200,300,400,600,700&amp;subset=latin-ext" rel="stylesheet">

        <!-- CSS WebSlides -->
        <link rel="stylesheet" type='text/css' media='all' href="static/css/webslides.css">

        <!-- Optional - CSS SVG Icons (Font Awesome) -->
        <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.15.1/css/all.min.css" integrity="sha512-+4zCK9k+qNFUR5X+cKL9EIR+ZOhtIloNl9GIKS57V1MyNsYpYcUrUeQc9vNfzsWfV28IaLL3i96P9sdNyeRssA==" crossorigin="anonymous" />

        <!-- SOCIAL CARDS (ADD YOUR INFO) -->

        <!-- FACEBOOK -->
        <meta property="og:url" content="{self.url}">
        <meta property="og:type" content="article">
        <meta property="og:title" content="{self.title}"> 
        <meta property="og:description" content="{self.description}">
        <meta property="og:updated_time" content="{datetime.today()}">
        <meta property="og:image" content="{self.image if self.image else "static/images/share-webslides.jpg"}">

        <!-- TWITTER -->
        <meta name="twitter:card" content="summary_large_image">
        <meta name="twitter:title" content="{self.title}"> 
        <meta name="twitter:description" content="{self.description}"> 
        <meta name="twitter:image" content="{self.image if self.image else "static/images/share-webslides.jpg"}">

        {self._generate_favicon_markup()}

        <!-- Android -->
        <meta name="mobile-web-app-capable" content="yes">
        <meta name="theme-color" content="#f0f0f0">

        <!-- Code highlighting -->
        <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/highlight.js/10.4.0/styles/default.min.css">
        <script src="https://cdnjs.cloudflare.com/ajax/libs/highlight.js/10.4.0/highlight.min.js"></script>
        <script>hljs.initHighlightingOnLoad();</script>

    </head>
    {self.navbar.__html__() if self.navbar else ""}
    <body>

    <main role='main'>
        <article id='webslides' {'class="vertical"' if self.vertical else ""}>
{self._generate_intro_slide()}
{slides_html}
{self._generate_endcard()}

        </article>
        <!-- end article -->
    </main>
    <!-- end main -->

    <!-- Required -->
    <script src='static/js/webslides.js'></script>
    <script>
        window.ws = new WebSlides();
    </script>

    <!-- OPTIONAL - svg-icons.js (fontastic.me - Font Awesome as svg icons) -->
    <script defer src='static/js/svg-icons.js'></script>

    </body>
    {self.footer.__html__() if self.footer else ""}
</html>
        '''
        return result


    def export(self, file_path:str, folder_name:Union[str, bool] = False, force:bool = False):
        """Exports the presentation files

        Parameters
        ----------
        file_path : (str)
            The path to export the folder of content to

        folder_name : (str or False)
            If you want to overwrite the folder name the output files are put in (defaults to Presentation.title), optional and defaults to False

        force : (bool)
            Whether to force generating files (overwrite existing files if found), optional and defaults to False

        Notes
        -----
        - all files are exported to file_path/folder_name
        - folder_name defaults to Presenation.title

        Raises
        ------
        FileExistsError
            If force is False, and a folder exists at file_path/folder_name

        Examples
        --------
        ### Export a presentation to the current directory at /Presentation
        ```
        from ezprez.core import Presentation
        prez = Presentation(title, description, url)

        # Export the files to the current directory at /Presentation
        prez.export(".", force=True, folder_name="Presentation")
        ```
        """
        if not folder_name:
            folder_name = self.title
        file_path = os.path.abspath(file_path)

        # finding downloads folder
        if os.name == "nt":
            DOWNLOAD_FOLDER = f"{os.getenv('USERPROFILE')}\\Downloads"
        else: # PORT: Assuming variable is there for MacOS and Linux installs
            DOWNLOAD_FOLDER = f"{os.getenv('HOME')}/Downloads" 

        # check if webslides is downloaded, if it is use those files, if not download and extract it
        if not os.path.exists(os.path.join(os.path.dirname(__file__), "webslides")):
            try:
                build(ZIPResource("webslides", "https://webslides.tv/webslides-latest.zip", overwrite_agreement=True))
                copytree(os.path.join(DOWNLOAD_FOLDER, "webslides"), os.path.join(os.path.dirname(__file__), "webslides"))
            except PermissionError:
                elevate()
                copytree(os.path.join(DOWNLOAD_FOLDER, "webslides"), os.path.join(os.path.dirname(__file__), "webslides"))
        try:
            copytree(os.path.join(os.path.dirname(__file__), "webslides"), os.path.join(file_path, folder_name))
        except FileExistsError:
            if force:
                rmtree(os.path.join(file_path, folder_name))
                copytree(os.path.join(os.path.dirname(__file__), "webslides"), os.path.join(file_path, folder_name))
            else:
                raise FileExistsError(f"The file path {os.path.join(file_path, folder_name)} exists, to replace use Presentation.export({file_path}, force=True)")

        # Copy image files
        if os.path.exists("./img"):
            for file_name in os.listdir("./img"):
                if os.path.isfile(os.path.join("img", file_name)):
                    copyfile(os.path.join("img", file_name), os.path.join(file_path, folder_name, "static", "images", file_name))
        elif os.path.exists("./images"):
            for file_name in os.listdir("./images"):
                if os.path.isfile(os.path.join("images", file_name)):
                    copyfile(os.path.join("images", file_name), os.path.join(file_path, folder_name, "static", "images", file_name))

        # replace index.html with generated html
        presentation_content = self.__html__()
        index_path = os.path.join(file_path, folder_name, "index.html")
        print(f"Writing html to {index_path}")
        with open(index_path, "w+") as presentation_file:
            presentation_file.write(presentation_content)



"""An object based api for generating web presentations

Note
----
If this is your first time using ezprez I would recommend looking at the user docs at https://ezprez.readthedocs.io

Installation
------------
#### From pypi
```pip install ezprez``` or ```sudo pip3 install ezprez```

#### From source
1. ```git clone https://github.com/Descent098/ezprez```
2. ```pip install .``` or ```sudo pip3 install .```

Modules
-------
#### core

The module that contains the Presentation and Slide classes that are used to generate web presentations

#### components

The module that contains all component subclasses that can be used to generate Slide content

Quickstart
----------
#### Creating a presentation with a text slide and exporting it to ./Presentation
```
from ezprez.core import Presentation, Slide

# This slide is auto-added to the Presentation object by default
Slide('This is the title', 'and this is the content')

prez = Presentation(title, description, url)

# Export the files to the current directory at /Presentation, and delete existing files if they're found
prez.export(".", force=True, folder_name="Presentation")
```

#### Creating a presentation with a slide that has text, and a code snippet and exporting it to ./Presentation
```
from ezprez.core import Presentation, Slide
from ezprez.components import Code

# This slide is auto-added to the Presentation object by default
Slide('This is the title', 'and here is some python code', Code('python', 'Hello World'))

prez = Presentation(title, description, url)

# Export the files to the current directory at /Presentation, and delete existing files if they're found
prez.export(".", force=True, folder_name="Presentation")
```
"""